#!/bin/sh
#BK may 2011
#i was inspired by Iguleder to patch the kernel to always be version number
#2.6.32 regardless of minor-version, so that we only have to compile the
#3rd-party drivers once.
#this script automates the kernel patching, compile, also creates a src .sfs.
#110710 KERNELVERSION variable.
#110718 create PET package.
#110805 now 3 scripts, run in order 1kernel.sh, 2modules.sh, 3package.sh
#110913 2.6.39 kernel.
#111003 bug fix.
#111014 3.0.4 kernel.
#120125 2.6.32.55 kernel.
#120324 3.2.13 kernel.
#120325 now ask for full kernel version number.
#130220 fix for 3.8.0, no linux patch. aufs include files fix.
#141018 K3.16.6, printk patch now in printk.h, previously was printk.c...
#140712 updated for kernel 4.x.
#161115 device-mapper patches for kernel boot params.
#170304 test if usb-storage patch exists.
#170321 remove full-install test.
#170918 create /boot if not exist.
#171219 k4.14+ /lib/firmware no longer created.
#180520 cap_sys_mount patch.
#190702 support aufs5

. /etc/DISTRO_SPECS

#KERNELVERSION='3.2' #110710 111014 120125 120324
#KERNELVER3="$KERNELVERSION"
#MINORVERSION='13'
#echo -n "Type in the kernel minor version [13]: "
#read newminor
#[ "$newminor" != "" ] && MINORVERSION="$newminor"
echo -n "Please type in the kernel version [4.1.2]: "
read fullkernelver
[ "$fullkernelver" = "" ] && fullkernelver="4.1.2"
KERNELVERSION="`echo -n "$fullkernelver" | rev | cut -f 2-9 -d '.' | rev`" #ex: 4.1
KERNELVER3="$KERNELVERSION"
MINORVERSION="`echo -n "$fullkernelver" | rev | cut -f 1 -d '.' | rev`" #ex: 2

TODAYDATE="`date +%Y%m%d`"

CURDIR="`pwd`"
if [ "$CURDIR" != "/usr/src" ];then
 echo 'This script must be in /usr/src'
 exit
fi
if [ ! -d 3rd-party/filesystem ];then
 echo 'Please download the aufs4-standalone*.tar.gz to 3rd-party/filesystem'
 echo 'and all of the other 3rd-party drivers, in correct subdirectories.'
 exit
fi
if [ ! -d 3rd-party/modems ];then
 echo 'Please download the drivers in 3rd-party/modems.'
 exit
fi
if [ ! -d 3rd-party/wireless ];then
 echo 'Please download the drivers in 3rd-party/wireless'
 exit
fi
if [ ! -f linux-${KERNELVER3}.tar.bz2 ];then
 echo "Please download linux-${KERNELVER3}.tar.bz2"
 exit
fi
if [ ! -f DOTconfig ];then
 echo 'Please download DOTconfig, the kernel configuration file'
 echo 'If you have another file that you want to use, rename it to DOTconfig'
 exit
fi

#echo
#echo "This script requires that you be running a full installation of Puppy."
#if [ -f /boot/vmlinuz ];then
# echo -n "This does appear to be the case, so just press ENTER to continue: "
# read YESGO
#else
# echo "It seems that you are not, so aborting."
# exit
#fi

echo
echo "This script is hardcoded to compile kernel ${KERNELVER3}.x"
echo "To change that, edit variable KERNELVERSION in script."
echo "(the 'x' is the kernel minor version, chosen below)"
echo

if [ "$MINORVERSION" != "0" ];then #130220
if [ ! -f patches/patch-${KERNELVER3}.${MINORVERSION}.bz2 ];then
 echo "Please download patch-${KERNELVER3}.${MINORVERSION}.bz2 and other patches, to patches dir"
 exit
fi
fi

rm -rf temp1 2>/dev/null
mkdir temp1

while [ 1 ];do
 echo
 echo "1:"
 echo "Would you prefer the kernel to install to lib/modules/${KERNELVER3}.${MINORVERSION}?"
 echo "This is the default, press ENTER only for this option."
 echo "2:"
 echo "Or, do you want the kernel to have version number ${KERNELVER3}, that is,"
 echo "install to /lib/modules/${KERNELVER3}, even though it has a minor version"
 echo "number ${MINORVERSION}? This may allow earlier 3rd-party kernel drivers to"
 echo "work after a kernel upgrade, without having to recompile the drivers."
 echo "Type '2' then ENTER for this option."
 echo "3:"
 echo "Or, would you like the kernel to have a unique version identifier, example"
 echo "'-uni', where the modules would be in /lib/modules/2.6.32-uni, (to keep same"
 echo "name for .44, .45 etc minor versions) or /lib/modules/2.6.32.${MINORVERSION}-uni?"
 echo "Type the string, ex1 '-uni', ex2 '.${MINORVERSION}-uni' then ENTER..."
 echo 
 echo "Note, you will be asked to verify whatever you type here..."
 echo "***OCT 2011: CURRENTLY BROKEN, CHOOSE [1] ONLY***"
 echo -n "Press ENTER only for the first option [1]: "
 read VERPATCH
 [ "$VERPATCH" = "" ] && VERPATCH="1"
 case $VERPATCH in
  1) #default, normal behaviour.
   echo
   echo "Are you sure about this? The kernel will identify itself as version"
   echo "'${KERNELVER3}.${MINORVERSION}' and the modules will be in /lib/modules/${KERNELVER3}.${MINORVERSION}."
   echo -n "ENTER only for okay: "
   read itisnotok
   [ "$itisnotok" ] && continue
   KERNELVERSION=${KERNELVER3}.${MINORVERSION}
   POSTVER=""
   EXTRAVERSION=".${MINORVERSION}"
  ;;
  2)
   echo
   echo "Are you sure about this? The kernel will identify itself as version"
   echo "'${KERNELVER3}' and the modules will be in /lib/modules/${KERNELVER3}."
   echo -n "ENTER only for okay (else any other char then ENTER): "
   read itisnotok
   [ "$itisnotok" ] && continue
   POSTVER="-${MINORVERSION}"
   EXTRAVERSION=""
  ;;
  *)
   echo
   echo "Are you sure about this? The kernel will identify itself as version"
   echo "'${KERNELVER3}${VERPATCH}' and the modules will be in /lib/modules/${KERNELVER3}${VERPATCH}."
   echo -n "ENTER only for okay (else any other char then ENTER): "
   read itisnotok
   [ "$itisnotok" ] && continue
   echo
   echo "The source SFS and binary PET packages that get built later can also have"
   echo "'-${MINORVERSION}' inserted in their name for subversion distinction, if"
   echo "you want. This is only desired if your kernel name '${KERNELVER3}${VERPATCH}'"
   echo "does not identify which subversion of the kernel is being built."
   echo -n "ENTER only to insert '-${MINORVERSION}' in pkg name (else any other char): "
   read insertminor
   if [ "$insertminor" ];then #111003
    POSTVER=""
   else
    POSTVER="-${MINORVERSION}"
   fi
   KERNELVERSION="${KERNELVER3}${VERPATCH}"
   EXTRAVERSION="${VERPATCH}"
  ;;
 esac
 break
done
echo "...ok, kernel version is '${KERNELVERSION}'"

rm -rf out-k* #get rid of earlier builds.

#if [ "$VERPATCH" ];then
# [ -d out-k${KERNELVER3}.${MINORVERSION} ] && rm -rf out-k${KERNELVER3}.${MINORVERSION}
# POSTVER="-${MINORVERSION}"
#else
# [ -d out-k${KERNELVER3}-${MINORVERSION} ] && rm -rf out-k${KERNELVER3}-${MINORVERSION}
# KERNELVERSION=${KERNELVER3}.${MINORVERSION}
# POSTVER=""
#fi

#patch kernel
echo
echo 'Extracting and patching kernel...'
rm -rf linux-${KERNELVER3} 2> /dev/null
rm -rf linux-${KERNELVERSION} 2> /dev/null
sync
tar -jxf linux-${KERNELVER3}.tar.bz2
sync

if [ "$MINORVERSION" != "0" ];then #130220
 echo
 echo "minor version number .${MINORVERSION} patch..."
 cp -f patches/patch-${KERNELVER3}.${MINORVERSION}.bz2 ./
 bunzip2 patch-${KERNELVER3}.${MINORVERSION}.bz2
 sync

 #if [ "$POSTVER" ];then
  #remove the .40 off end, so version number stays at 2.6.32...
  eREGEX="s%^\\+EXTRAVERSION \\= .*%\\+EXTRAVERSION \\= ${EXTRAVERSION}%"
  sed -i -e "$eREGEX" patch-${KERNELVER3}.${MINORVERSION}
 #fi

 cd linux-${KERNELVER3}
 patch -p1 < ../patch-${KERNELVER3}.${MINORVERSION}
 cd ..
 sync
 rm patch-${KERNELVER3}.${MINORVERSION}
fi

#[ "$POSTVER" = "" ] && mv -f linux-${KERNELVER3} linux-${KERNELVER3}.${MINORVERSION}
[ "${KERNELVER3}" != "${KERNELVERSION}" ] && mv -f linux-${KERNELVER3} linux-${KERNELVERSION}
ln -snf linux-${KERNELVERSION} linux

#loglevel
echo
if [ -f patches/printk.c.patch.gz ];then
 echo 'loglevel patch...'
 cp -f patches/printk.c.patch.gz ./
 gunzip printk.c.patch.gz
 cd linux-${KERNELVERSION}
 patch -p0 < ../printk.c.patch
 cd ..
 rm printk.c.patch
fi
#141018 K3.16.6, printk patch now in printk.h, previously was printk.c...
if [ -f patches/printk.h.patch.gz ];then
 echo 'loglevel patch...'
 cp -f patches/printk.h.patch.gz ./
 gunzip printk.h.patch.gz
 cd linux-${KERNELVERSION}
 patch -p0 < ../printk.h.patch
 cd ..
 rm printk.h.patch
fi

#vortex86mx cpu
echo
if [ -f patches/vortex86mx.patch.gz ];then
 echo 'vortex86mx cpu patch...'
 cp -f patches/vortex86mx.patch.gz ./
 gunzip vortex86mx.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../vortex86mx.patch
 cd ..
 rm vortex86mx.patch
fi

#aufs2.1
echo
if [ "`ls 3rd-party/filesystem/aufs2-standalone*.tar.gz 2>/dev/null`" != "" ];then
 echo 'aufs patch...'
 cp -f 3rd-party/filesystem/aufs2-standalone*.tar.gz temp1/aufs.tar.gz
 cd temp1
 tar -zxf aufs.tar.gz
 cd aufs2-standalone
 cp -f aufs2-base.patch ../../
 cp -f aufs2-kbuild.patch ../../
 cp -f aufs2-standalone.patch ../../
 cd ../../linux-${KERNELVERSION}
 patch -p1 < ../aufs2-kbuild.patch
 patch -p1 < ../aufs2-base.patch
 patch -p1 < ../aufs2-standalone.patch
 cp -a -f ../temp1/aufs2-standalone/Documentation ./
 cp -a -f ../temp1/aufs2-standalone/fs ./
 cp -a -f ../temp1/aufs2-standalone/include/linux/aufs_type.h ./include/linux/
 cd ..
 gzip aufs2-kbuild.patch
 gzip aufs2-base.patch
 gzip aufs2-standalone.patch
 mv -f aufs2-kbuild.patch.gz patches/
 mv -f aufs2-base.patch.gz patches/
 mv -f aufs2-standalone.patch.gz patches/
 
 #aufs-allow-sfs-in-branch
 echo
 echo 'aufs-allow-sfs-in-branch patch...'
 cp patches/aufs-allow-sfs.patch.gz ./
 gunzip aufs-allow-sfs.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../aufs-allow-sfs.patch
 cd ..
 rm aufs-allow-sfs.patch
fi

if [ "`ls 3rd-party/filesystem/aufs3*.tar.gz 2>/dev/null`" != "" ];then
 echo 'aufs patch...'
 cp -f 3rd-party/filesystem/aufs3*.tar.gz temp1/aufs.tar.gz
 cd temp1
 tar -zxf aufs.tar.gz
 DIRNAME="`find -mindepth 1 -maxdepth 1 -type d -name 'aufs3*'`"
 cd $DIRNAME
 cp -f aufs3-base.patch ../../
 cp -f aufs3-kbuild.patch ../../
 cp -f aufs3-standalone.patch ../../
 #cp -f aufs3-loopback.patch ../../
 cp -f aufs3-mmap.patch ../../ #131204
 cd ../../linux-${KERNELVERSION}
 patch -p1 < ../aufs3-kbuild.patch
 patch -p1 < ../aufs3-base.patch
 patch -p1 < ../aufs3-standalone.patch
 #patch -p1 < ../aufs3-loopback.patch
 patch -p1 < ../aufs3-mmap.patch #131204
 cp -a -f ../temp1/$DIRNAME/Documentation ./
 cp -a -f ../temp1/$DIRNAME/fs ./
 cp -a -f ../temp1/$DIRNAME/include/linux/aufs_type.h ./include/linux/ 2>/dev/null #131204 fails.
 [ -d ../temp1/$DIRNAME/include/uapi/linux ] && cp -a -f ../temp1/$DIRNAME/include/uapi/linux/aufs_type.h ./include/uapi/linux/ #130220
 cd ..
 gzip aufs3-kbuild.patch
 gzip aufs3-base.patch
 gzip aufs3-standalone.patch
 gzip aufs3-mmap.patch
 mv -f aufs3-kbuild.patch.gz patches/
 mv -f aufs3-base.patch.gz patches/
 mv -f aufs3-standalone.patch.gz patches/
 mv -f aufs3-mmap.patch.gz patches/
 #mv -f aufs3-loopback.patch.gz patches/
 
 #aufs-allow-sfs-in-branch
 echo
 echo 'aufs-allow-sfs-in-branch patch...'
 cp patches/aufs-allow-sfs.patch.gz ./
 gunzip aufs-allow-sfs.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../aufs-allow-sfs.patch
 cd ..
 rm aufs-allow-sfs.patch
fi

if [ "`ls 3rd-party/filesystem/aufs4*.tar.gz 2>/dev/null`" != "" ];then
 echo 'aufs patch...'
 cp -f 3rd-party/filesystem/aufs4*.tar.gz temp1/aufs.tar.gz
 cd temp1
 tar -zxf aufs.tar.gz
 DIRNAME="`find -mindepth 1 -maxdepth 1 -type d -name 'aufs4*'`"
 cd $DIRNAME
 cp -f aufs4-base.patch ../../
 cp -f aufs4-kbuild.patch ../../
 cp -f aufs4-standalone.patch ../../
 #cp -f aufs4-loopback.patch ../../
 cp -f aufs4-mmap.patch ../../ #131204
 cd ../../linux-${KERNELVERSION}
 patch -p1 < ../aufs4-kbuild.patch
 patch -p1 < ../aufs4-base.patch
 patch -p1 < ../aufs4-standalone.patch
 #patch -p1 < ../aufs4-loopback.patch
 patch -p1 < ../aufs4-mmap.patch #131204
 cp -a -f ../temp1/$DIRNAME/Documentation ./
 cp -a -f ../temp1/$DIRNAME/fs ./
 cp -a -f ../temp1/$DIRNAME/include/linux/aufs_type.h ./include/linux/ 2>/dev/null #131204 fails.
 [ -d ../temp1/$DIRNAME/include/uapi/linux ] && cp -a -f ../temp1/$DIRNAME/include/uapi/linux/aufs_type.h ./include/uapi/linux/ #130220
 cd ..
 gzip aufs4-kbuild.patch
 gzip aufs4-base.patch
 gzip aufs4-standalone.patch
 gzip aufs4-mmap.patch
 mv -f aufs4-kbuild.patch.gz patches/
 mv -f aufs4-base.patch.gz patches/
 mv -f aufs4-standalone.patch.gz patches/
 mv -f aufs4-mmap.patch.gz patches/
 #mv -f aufs4-loopback.patch.gz patches/
 
 #aufs-allow-sfs-in-branch
 echo
 echo 'aufs-allow-sfs-in-branch patch...'
 cp patches/aufs-allow-sfs.patch.gz ./
 gunzip aufs-allow-sfs.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../aufs-allow-sfs.patch
 cd ..
 rm aufs-allow-sfs.patch
fi

#20190702 aufs5
if [ "`ls 3rd-party/filesystem/aufs5*.tar.gz 2>/dev/null`" != "" ];then
 echo 'aufs patch...'
 cp -f 3rd-party/filesystem/aufs5*.tar.gz temp1/aufs.tar.gz
 cd temp1
 tar -zxf aufs.tar.gz
 DIRNAME="`find -mindepth 1 -maxdepth 1 -type d -name 'aufs5*'`"
 cd $DIRNAME
 cp -f aufs5-base.patch ../../
 cp -f aufs5-kbuild.patch ../../
 cp -f aufs5-standalone.patch ../../
 #cp -f aufs5-loopback.patch ../../
 cp -f aufs5-mmap.patch ../../ #131204
 cd ../../linux-${KERNELVERSION}
 patch -p1 < ../aufs5-kbuild.patch
 patch -p1 < ../aufs5-base.patch
 patch -p1 < ../aufs5-standalone.patch
 #patch -p1 < ../aufs5-loopback.patch
 patch -p1 < ../aufs5-mmap.patch #131204
 cp -a -f ../temp1/$DIRNAME/Documentation ./
 cp -a -f ../temp1/$DIRNAME/fs ./
 cp -a -f ../temp1/$DIRNAME/include/linux/aufs_type.h ./include/linux/ 2>/dev/null #131204 fails.
 [ -d ../temp1/$DIRNAME/include/uapi/linux ] && cp -a -f ../temp1/$DIRNAME/include/uapi/linux/aufs_type.h ./include/uapi/linux/ #130220
 cd ..
 gzip aufs5-kbuild.patch
 gzip aufs5-base.patch
 gzip aufs5-standalone.patch
 gzip aufs5-mmap.patch
 mv -f aufs5-kbuild.patch.gz patches/
 mv -f aufs5-base.patch.gz patches/
 mv -f aufs5-standalone.patch.gz patches/
 mv -f aufs5-mmap.patch.gz patches/
 #mv -f aufs5-loopback.patch.gz patches/
 
 #aufs-allow-sfs-in-branch
 echo
 echo 'aufs-allow-sfs-in-branch patch...'
 cp patches/aufs-allow-sfs.patch.gz ./
 gunzip aufs-allow-sfs.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../aufs-allow-sfs.patch
 cd ..
 rm aufs-allow-sfs.patch
fi



#commoncap security
echo
if [ -f patches/commoncap-symbol.patch.gz ];then
 echo 'commoncap symbol export patch...'
 cp patches/commoncap-symbol.patch.gz ./
 gunzip commoncap-symbol.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../commoncap-symbol.patch
 cd ..
 rm commoncap-symbol.patch
fi

#161115
if [ -f patches/01-dm-v5-1-3.patch.gz ];then
 echo 'device-mapper patches...'
 cp -f patches/01-dm-v5-1-3.patch.gz ./
 gunzip 01-dm-v5-1-3.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../01-dm-v5-1-3.patch
 cd ..
 rm -f 01-dm-v5-1-3.patch
 cp -f patches/02-dm-v5-2-3.patch.gz ./
 gunzip 02-dm-v5-2-3.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../02-dm-v5-2-3.patch
 cd ..
 rm -f 02-dm-v5-2-3.patch
 cp -f patches/03-dm-v5-3-3.patch.gz ./
 gunzip 03-dm-v5-3-3.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../03-dm-v5-3-3.patch
 cd ..
 rm -f 03-dm-v5-3-3.patch
fi

#usb-storage
if [ -f patches/usb-storage.patch.gz ];then #170304
 if vercmp ${KERNELVERSION} ge 2.6.33;then
  echo
  echo 'usb-storage patch...'
  cp patches/usb-storage.patch.gz ./
  gunzip usb-storage.patch.gz
  cd linux-${KERNELVERSION}
  if vercmp ${KERNELVERSION} ge 3.0.25;then
   patch -p1 < ../usb-storage.patch
  else
   patch -p0 < ../usb-storage.patch
  fi
  cd ..
  rm usb-storage.patch
 fi
fi

#unionfs
if [ "`ls patches/unionfs*.gz 2>/dev/null`" != "" ];then
 cp -f patches/unionfs*.gz unionfs.patch.gz
 gunzip unionfs.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../unionfs.patch
 cd ..
 rm unionfs.patch
fi

#180520
if [ -f patches/cap_sys_mount-1.patch.gz ];then
 echo
 echo "cap_sys_mount patch..."
 cp -f patches/cap_sys_mount-1.patch.gz ./
 gunzip cap_sys_mount-1.patch.gz
 cp -f patches/cap_sys_mount-2.patch.gz ./
 gunzip cap_sys_mount-2.patch.gz
 cd linux-${KERNELVERSION}
 patch -p1 < ../cap_sys_mount-1.patch
 patch -p1 < ../cap_sys_mount-2.patch
 cd ..
 rm cap_sys_mount-1.patch
 rm cap_sys_mount-2.patch
fi

#create tarball of patched source
sync
echo
echo 'Creating kernel patched source tarball...'
rm -f linux-${KERNELVERSION}${POSTVER}-src-patched.tar.bz2 2>/dev/null
tar -c -f linux-${KERNELVERSION}${POSTVER}-src-patched.tar linux-${KERNELVERSION}/
bzip2 linux-${KERNELVERSION}${POSTVER}-src-patched.tar
sync

#compile kernel
echo
echo -n 'ENTER to compile kernel: '
read goforit
cd linux-${KERNELVERSION}
make clean
make mrproper
cp -f ../DOTconfig .config
make menuconfig
sync
sleep 1
cp -f .config DOTconfig-${KERNELVERSION}${POSTVER}
cp -f DOTconfig-${KERNELVERSION}${POSTVER} ../
echo -n 'ENTER to compile kernel: '
read goforit
make
sync

#place to send everything...
rm -rf /usr/src/out-k${KERNELVERSION}${POSTVER} 2> /dev/null
mkdir /usr/src/out-k${KERNELVERSION}${POSTVER}


#install kernel
echo
echo -n "ENTER to install kernel: "
read goforit
echo
echo "Installing modules..."
rm -rf /lib/modules/${KERNELVERSION} 2> /dev/null
rm -rf /lib/firmware-OLD 2> /dev/null
mv -f /lib/firmware /lib/firmware-OLD
mkdir /lib/firmware #171219
make modules_install
sync
echo "Creating 'firmware' tarball in /usr/src/out-k${KERNELVERSION}${POSTVER}..."
cp -a -f /lib/firmware ../out-k${KERNELVERSION}${POSTVER}/
echo "Creating headers tarball in /usr/src/out-k${KERNELVERSION}${POSTVER}..."
make headers_install
sync
mkdir -p /usr/src/out-k${KERNELVERSION}${POSTVER}/linux_headers-${KERNELVERSION}${POSTVER}/usr/include
cp -a -f usr/include/* ../out-k${KERNELVERSION}${POSTVER}/linux_headers-${KERNELVERSION}${POSTVER}/usr/include/
sync

#120325 turn headers into a pet...
#linux_headers_DEV-2.6.32-40|linux_headers_DEV|2.6.32-40||BuildingBlock|4236K||linux_headers_DEV-2.6.32-40.pet||header files for 2.6.32.40 kernel||||
HDRSIZE=$(du -s -k ../out-k${KERNELVERSION}${POSTVER}/linux_headers-${KERNELVERSION}${POSTVER} | cut -f 1)
echo "linux_headers-${KERNELVERSION}${POSTVER}|linux_headers|${KERNELVERSION}${POSTVER}||BuildingBlock|${HDRSIZE}K||linux_headers-${KERNELVERSION}${POSTVER}.pet||header files for ${KERNELVERSION}${POSTVER} kernel||||" > ../out-k${KERNELVERSION}${POSTVER}/linux_headers-${KERNELVERSION}${POSTVER}/pet.specs


echo "Creating tarball of modules..."
cp -a -f /lib/modules/${KERNELVERSION} ../out-k${KERNELVERSION}${POSTVER}/

echo "Copying DOTconfig-${KERNELVERSION}${POSTVER}, System.map, vmlinuz..."
cp -f DOTconfig-${KERNELVERSION}${POSTVER} ../out-k${KERNELVERSION}${POSTVER}/
cp -f System.map ../out-k${KERNELVERSION}${POSTVER}/
cp -f modules.order ../out-k${KERNELVERSION}${POSTVER}/
cp -f modules.builtin ../out-k${KERNELVERSION}${POSTVER}/ 2>/dev/null #<=2.6.32 does not have this.
cp -f arch/x86/boot/bzImage ../out-k${KERNELVERSION}${POSTVER}/vmlinuz
sync
cd ../out-k${KERNELVERSION}${POSTVER}
dir2tgz firmware/
dir2tgz linux_headers-${KERNELVERSION}${POSTVER}
sync
tgz2pet linux_headers-${KERNELVERSION}${POSTVER}.tar.gz
dir2tgz ${KERNELVERSION}
sync
rm -rf firmware
rm -rf linux_headers-${KERNELVERSION}${POSTVER}
rm -rf ${KERNELVERSION}
cd ..
cp -f -a patches out-k${KERNELVERSION}${POSTVER}/
cp -f -a 3rd-party out-k${KERNELVERSION}${POSTVER}/
mv -f DOTconfig DOTconfig-PREVIOUS
cp -f -a DOTconfig-${KERNELVERSION}${POSTVER} DOTconfig
sync

mv -f linux-${KERNELVERSION}${POSTVER}-src-patched.tar.bz2 out-k${KERNELVERSION}${POSTVER}/
sync

echo
echo "Finished!"
echo "The next step is to run '2modules.sh' to compile 3rd-party modules."
echo "However, Puppy should be running and this will require a reboot."
echo "Just press ENTER to copy 'vmlinuz' to /boot"
echo "This script will then exit, and you will have to reboot."
echo -n "Press ENTER: "
read YESGO
if [ "$YESGO" = "" ];then
 OLDKER="`uname -r`"
 mkdir -p /boot
 [ -f /boot/vmlinuz ] && mv -f /boot/vmlinuz /boot/vmlinuz-${OLDKER}-OLD
 cp -f out-k${KERNELVERSION}${POSTVER}/vmlinuz /boot/
 echo "Copied 'vmlinuz' to /boot, now you must reboot."
fi

echo "1kernel.sh finished"
###END###
